package sun.util;

import java.io.IOException;
import java.io.ObjectInputStream;
import java.util.Map;
import sun.util.locale.provider.CalendarDataUtility;
import java.util.Locale;
import java.util.TimeZone;
import java.util.GregorianCalendar;

public class BuddhistCalendar extends GregorianCalendar
{
    private static final long serialVersionUID = -8527488697350388578L;
    private static final int BUDDHIST_YEAR_OFFSET = 543;
    private transient int yearOffset;
    
    public BuddhistCalendar() {
        this.yearOffset = 543;
    }
    
    public BuddhistCalendar(TimeZone zone) {
        super(zone);
        this.yearOffset = 543;
    }
    
    public BuddhistCalendar(Locale aLocale) {
        super(aLocale);
        this.yearOffset = 543;
    }
    
    public BuddhistCalendar(TimeZone zone, Locale aLocale) {
        super(zone, aLocale);
        this.yearOffset = 543;
    }
    
    @Override
    public String getCalendarType() {
        return "buddhist";
    }
    
    @Override
    public boolean equals(Object obj) {
        return obj instanceof BuddhistCalendar && super.equals(obj);
    }
    
    @Override
    public int hashCode() {
        return super.hashCode() ^ 0x21F;
    }
    
    @Override
    public int get(int n) {
        if (n == 1) {
            return super.get(n) + this.yearOffset;
        }
        return super.get(n);
    }
    
    @Override
    public void set(int n, int value) {
        if (n == 1) {
            super.set(n, value - this.yearOffset);
        }
        else {
            super.set(n, value);
        }
    }
    
    @Override
    public void add(int field, int amount) {
        int yearOffset = this.yearOffset;
        this.yearOffset = 0;
        try {
            super.add(field, amount);
        }
        finally {
            this.yearOffset = yearOffset;
        }
    }
    
    @Override
    public void roll(int field, int amount) {
        int yearOffset = this.yearOffset;
        this.yearOffset = 0;
        try {
            super.roll(field, amount);
        }
        finally {
            this.yearOffset = yearOffset;
        }
    }
    
    @Override
    public String getDisplayName(int n, int n2, Locale locale) {
        if (n != 0) {
            return super.getDisplayName(n, n2, locale);
        }
        return CalendarDataUtility.retrieveFieldValueName("buddhist", n, this.get(n), n2, locale);
    }
    
    @Override
    public Map<String, Integer> getDisplayNames(int n, int n2, Locale locale) {
        if (n != 0) {
            return super.getDisplayNames(n, n2, locale);
        }
        return CalendarDataUtility.retrieveFieldValueNames("buddhist", n, n2, locale);
    }
    
    @Override
    public int getActualMaximum(int field) {
        int yearOffset = this.yearOffset;
        this.yearOffset = 0;
        try {
            return super.getActualMaximum(field);
        }
        finally {
            this.yearOffset = yearOffset;
        }
    }
    
    @Override
    public String toString() {
        String string = super.toString();
        if (!this.isSet(1)) {
            return string;
        }
        int index = string.indexOf("YEAR=");
        if (index == -1) {
            return string;
        }
        int endIndex = index + "YEAR=".length();
        StringBuilder sb = new StringBuilder(string.substring(0, endIndex));
        while (Character.isDigit(string.charAt(endIndex++))) {}
        sb.append(this.internalGet(1) + 543).append(string.substring(endIndex - 1));
        return sb.toString();
    }
    
    private void readObject(ObjectInputStream objectInputStream) throws IOException, ClassNotFoundException {
        objectInputStream.defaultReadObject();
        this.yearOffset = 543;
    }
}
